////////////////////////////////////////////////////////////////////////////////
/*! \file       string_tokeniser.cpp
    \brief      Implementation file for string_tokeniser class
    \author     Orjan Westin [ orjan@coolcowstudio.co.uk ]
    \version    1.0
    \date       2012
    \copyright  BSD (see EOF comment)
    Download:   http://coolcowstudio.co.uk/software/source/cpp/utilities.zip
*///////////////////////////////////////////////////////////////////////////////

#include "string_tokeniser.h"

namespace CoCoSt
{
  namespace Utilities
  {
    // Constructor
    string_tokeniser::string_tokeniser(const std::string& source, bool empty /*= false*/)
      : source_(source)
      , current_(std::string::npos)
      , next_(source.empty() ? std::string::npos : 0)
      , length_(0)
      , empty_(empty)
    {}

    // Advance to next token
    bool string_tokeniser::next(const std::string::value_type& separator)
    {
      // Store the start
      current_ = next_;
      if (at_end())
        return false;
      // Find next
      next_ = source_.find(separator, current_);
      // Deal with result of search
      return handle_next(1, separator);
    }

    // Advance to next token
    bool string_tokeniser::next(const std::string& separator)
    {
      // Store the start
      current_ = next_;
      if (at_end())
        return false;
      // Find next
      next_ = source_.find(separator, current_);
      // Deal with result of search
      return handle_next(separator.size(), separator);
    }

    // Advance to next token
    bool string_tokeniser::next(const std::vector<char>& separators)
    {
      // Store the start
      current_ = next_;
      if (at_end())
        return false;
      // Find next
      next_ = source_.find_first_of(&separators[0], current_, separators.size());
      // Deal with result of search
      return handle_next(1, separators);
    }

    // Get current token, if any, safely
    bool string_tokeniser::get_token(std::string& token) const
    {
      if (!has_token())
        return false;
      if (0 == length_)
        token.clear();
      else
        token = source_.substr(current_, length_);
      return true;
    }

    // Get current token, if any, otherwise an empty token
    std::string string_tokeniser::get_token() const
    {
      if (!has_token() || (0 == length_))
        return std::string();
      return source_.substr(current_, length_);
    }

    // Reset search so it can be restarted
    void string_tokeniser::reset()
    {
      current_ = std::string::npos;
      next_ = source_.empty() ? std::string::npos : 0;
      length_ = 0;
    }

    // Return true if there is a current token
    bool string_tokeniser::has_token() const
    {
      // Not worried about length here, as it might be an empty token
      return (std::string::npos != current_);
    }

    // Return true if no further searches can be done
    bool string_tokeniser::at_end() const
    { 
      return (std::string::npos == next_);
    }

    // Get source string
    const std::string& string_tokeniser::get_source() const
    {
      return source_;
    }

    ////////////////////////////////////////////////////////////////////////////
    // Helper

    // Handle the result of a search, advancing to prepare for next search
    template <typename T>
    bool string_tokeniser::handle_next(size_t advance, const T& separator)
    {
      if (std::string::npos == next_)
      {
        // Separator not found, but there might still be data, at the end 
        length_ = source_.size() - current_;
      }
      else
      {
        // Store the length of the current token
        length_ = next_ - current_;
        // and move next starting point to beyond the one we found
        next_ += advance;
        // In the case of double separators (e.g. | in "a|b||d"), this gives an 
        // empty token. If empties aren't accepted, we'll recurse
        if ((0 == length_) && !empty_)
        {
          return next(separator);
        }
      }
      // Do we have a token?
      if (0 < length_)
        return true;
      // Even if empties are accepted, dismiss an empty token at the end of a 
      // string (e.g. "a|b|" gives "a" and "b" only)
      if (!empty_ || (std::string::npos == next_))
      {
        // Invalidate current, so extraction isn't valid
        current_ = next_;
        return false;
      }
      return true;    
    }
  }
}

/*

Copyright (c) 2012, Orjan Westin
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

The name of the contributor(s) may be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

