////////////////////////////////////////////////////////////////////////////////
/*! \file       string_tokeniser.h
    \brief      Header file for string_tokeniser class
    \author     Orjan Westin [ orjan@coolcowstudio.co.uk ]
    \version    1.0
    \date       2012
    \copyright  BSD (see EOF comment)
    Download:   http://coolcowstudio.co.uk/software/source/cpp/utilities.zip
*///////////////////////////////////////////////////////////////////////////////

#ifndef COCOST_STRING_TOKENISER_H_
#define COCOST_STRING_TOKENISER_H_

#include <string>
#include <vector>

//! Cool Cow Studio namespace
namespace CoCoSt 
{
  //! Utility function namespace
  namespace Utilities
  {
    //! String tokeniser class
    /*! Class to find and extract tokens from string cheaply and 
        non-destructively, using non-fixed delimiters, like strtok.
        Stores a const reference to the string to search, which means that if
        this string is modified between token searches, the search is invalid
        and the tokeniser needs to be reset.
    */
    class string_tokeniser
    {
      // The string we're searching in
      const std::string& source_;
      // Current location in string
      std::string::size_type current_;
      // Location to start next search
      std::string::size_type next_;
      // Length of current string
      std::string::difference_type length_;
      // Flag indicating whether to include empty strings
      bool empty_;

      // Helper - handle the result of a search, advancing to prepare for next
      template <typename T>
      bool handle_next(size_t advance, const T& separator);

    public:
      //! Constructor
      /*! Constructor, setting the string to work on
          After construction, a call to next() is required to find the first token
          \param source string to work on
          \param empty if true, next() will yield empty strings between two separators
      */
      string_tokeniser(const std::string& source, bool empty = false);

      //! Advance to next token
      /*! Advance to next token, by given character separator
          E.g "Smith&Jones", separated by '&' gives "Smith", then "Jones"
          \param separator character indicating end of token
          \return true if there is a token to get after the call
      */
      bool next(const std::string::value_type& separator);

      //! Advance to next token
      /*! Advance to next token, by given string separator
          E.g "Smith and Jones", separated by " and " gives "Smith", then "Jones"
          \param separator string indicating end of token
          \return true if there is a token to get after the call
      */
      bool next(const std::string& separator);

      //! Advance to next token
      /*! Advance to next token, by given separator
          E.g "Smith & Jones", separated by {'&',' ','-'} gives "Smith", then "Jones"
          (if empty flag not set in construction - otherwise two empty strings are 
          found between the two names).
          \param separators characters that can indicate end of token
          \return true if there is a token to get after the call
      */
      bool next(const std::vector<char>& separators);

      //! Get current token
      /*! Get current token, if any, safely
          \param token string to populate with current token, if any
          \return true if a token was available
      */
      bool get_token(std::string& token) const;

      //! Get current token
      /*! Get current token, if any, otherwise an empty token
          \return current token, if any, otherwise an empty string
      */
      std::string get_token() const;

      //! Reset search
      /*! Reset search so the string can be searched from the beginning.
          Required if the string has been modified between calls to next.
      */
      void reset();

      //! Check token availability
      /*! Check if a token is available
          /return true if there is a current token
      */
      bool has_token() const;

      //! Check if search is at end
      /*! Check if search is at end
         /return true if no further searches can be done
      */
      bool at_end() const;

      //! Get source string
      /*! Get source string
          \return reference to source string
      */
      const std::string& get_source() const;
    };
  }
}

#endif // COCOST_STRING_TOKENISER_H_

/*
Copyright (c) 2012, Orjan Westin
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

The name of the contributor(s) may be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

