////////////////////////////////////////////////////////////////////////////////
/*! \file       string_utils.h
    \brief      Implementation file for string utility functions
    \author     Orjan Westin [ orjan@coolcowstudio.co.uk ]
    \version    1.1
    \date       2010-2011
    \copyright  BSD (see EOF comment)
    Download:   http://coolcowstudio.co.uk/software/source/cpp/utilities.zip
*///////////////////////////////////////////////////////////////////////////////

#include "string_utils.h"
#include "static_assert.hpp"

#include <algorithm>
#include <climits>

namespace CoCoSt
{
  namespace Utilities
  {

    // Put all token substrings, divided by given separator, as individual strings in collection
    void tokenise_string(const std::string& str, const std::string& separator, std::vector<std::string>& tokens, bool empty)
    {
      const std::string::size_type strlength = str.length();
      const std::string::size_type seplength = separator.length();

      std::string::size_type prev = 0;
      std::string::size_type next = str.find(separator, prev);

      while (std::string::npos != next)
      {
        if (empty || prev != next)
          tokens.push_back(str.substr(prev, next - prev));
        prev = next + seplength;
        next = str.find(separator, prev);
      }
      if (empty || prev != strlength)
        tokens.push_back(str.substr(prev, strlength - prev));
    }

    // Put all token substrings, divided by given separator, as individual strings in collection
    void tokenise_string(const std::string& str, std::string::value_type separator, std::vector<std::string>& tokens, bool empty)
    {
      const std::string::size_type strlength = str.length();
      const std::string::size_type seplength = 1;

      std::string::size_type prev = 0;
      std::string::size_type next = str.find(separator, prev);

      while (std::string::npos != next)
      {
        if (empty || prev != next)
          tokens.push_back(str.substr(prev, next - prev));
        prev = next + seplength;
        next = str.find(separator, prev);
      }
      if (empty || prev != strlength)
        tokens.push_back(str.substr(prev, strlength - prev));
    }

    // Put all token substrings, divided by one of the given separators, as individual strings in collection
    void tokenise_string(const std::string& str, const std::vector<std::string::value_type>& separators, std::vector<std::string>& tokens, bool empty)
    {
      const std::string::size_type strlength = str.length();
      const std::string::size_type seplength = 1;
      const std::string sep(separators.begin(), separators.end());

      std::string::size_type prev = 0;
      std::string::size_type next = str.find_first_of(sep, prev);

      while (std::string::npos != next)
      {
        if (empty || prev != next)
          tokens.push_back(str.substr(prev, next - prev));
        prev = next + seplength;
        next = str.find_first_of(sep, prev);
      }
      if (empty || prev != strlength)
        tokens.push_back(str.substr(prev, strlength - prev));
    }

    // Put all token substrings, divided by one of the given separators, as individual strings in collection
    void merge_tokens(const std::vector<std::string> &tokens, const std::string& separator, std::string& output)
    {
      if (!tokens.empty())
      {
        output = tokens.front();
        for (std::vector<std::string>::const_iterator i = ++(tokens.begin()); i != tokens.end(); ++i)
          output += separator + *i;
      }
    }

    // Replace all occurences of one substring with another
    void replace_all_substrings(const std::string& pre, const std::string& post, std::string& str)
    {
      const std::string::size_type preLen = pre.length();
      const std::string::size_type postLen = post.length();

      std::string::size_type pos = str.find(pre);
      while (std::string::npos != pos)
      {
        str.replace(pos, preLen, post);
        pos = str.find(pre, pos + postLen);
      } 
    }

    // Remove all occurences of space, carriage return, line feed, tab etc as defined by current locale
    void remove_whitespace(std::string& str)
    {
      str.erase(std::remove_if(str.begin(), str.end(), &::isspace), str.end());
    }

    // Remove all occurences of space, carriage return, line feed, tab etc as defined by given locale
    void remove_whitespace(const std::string& input, std::string& output)
    {
      output.clear();
      std::remove_copy_if(input.begin(), input.end(), std::back_inserter(output), &::isspace);
    }
    
    // Convert string to upper case using current locale
    void toupper(std::string& str)
    { 
      std::transform(str.begin(), str.end(), str.begin(), ::toupper);
    }

    // Convert string to lower case using current locale
    void tolower(std::string& str)
    {
      std::transform(str.begin(), str.end(), str.begin(), ::tolower);
    }
  }
}

/*

Copyright (c) 2010-2011, Orjan Westin
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

The name of the contributor(s) may be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

