////////////////////////////////////////////////////////////////////////////////
/*! \file       string_utils.h
    \brief      Header file for string-based utility functions
    \author     Orjan Westin [ orjan@coolcowstudio.co.uk ]
    \version    1.1
    \date       2010-2011
    \copyright  BSD (see EOF comment)
    Download:   http://coolcowstudio.co.uk/software/source/cpp/utilities.zip
*///////////////////////////////////////////////////////////////////////////////

#ifndef COCOST_STRING_UTILS_H_
#define COCOST_STRING_UTILS_H_

#include <string>
#include <vector>

//! Cool Cow Studio namespace
namespace CoCoSt 
{
  //! Utility function namespace
  namespace Utilities
  {
    //! Divide a string into a collection of substrings, or tokens
    /*! Put all token substrings, divided by given separator, as individual 
        strings in collection
        \param str string of tokens to divide up
        \param separator character separating the tokens
        \param tokens collection receiving tokens
        \param empty indicates whether to include empty tokens (default false)
    */
    void tokenise_string(const std::string& str, 
            std::string::value_type separator, 
            std::vector<std::string>& tokens, 
            bool empty = false);

    //! Divide a string into a collection of substrings, or tokens
    /*! Put all token substrings, divided by given separator, as individual 
        strings in collection
        \param str string of tokens to divide up
        \param separator string separating the tokens
        \param tokens collection receiving tokens
        \param empty indicates whether to include empty tokens (default false)
    */
    void tokenise_string(const std::string& str, 
            const std::string& separator, 
            std::vector<std::string>& tokens, 
            bool empty = false);

    //! Divide a string into a collection of substrings, or tokens
    /*! Put all token substrings, divided by one of the given separators, as 
        individual strings in collection
        \param str string of tokens to divide up
        \param separators list of characters separating the tokens
        \param tokens collection receiving tokens
        \param empty indicates whether to include empty tokens (default false)
    */
    void tokenise_string(const std::string& str, 
            const std::vector<std::string::value_type>& separators, 
            std::vector<std::string>& tokens, 
            bool empty = false);

    //! Merge tokens into one string
    /*! Merge all given tokens into one string, each separated by given 
        separator
        \param tokens collection of tokens to merge
        \param separator string to separate the tokens
        \param output string receiving merged results
    */
    void merge_tokens(const std::vector<std::string> &tokens, 
            const std::string& separator, 
            std::string& output);
    
    //! Replace all occurences of one substring with another
    /*! Replace all occurences of one substring with another
        \param pre substring to replace
        \param post replacement string
        \param str string in which to do replacement
    */
    void replace_all_substrings(const std::string& pre, 
            const std::string& post, 
            std::string& str);
   
    //! Remove all whitespace characters from string
    /*! Remove all occurences of space, carriage return, line feed, tab etc as
        defined by current locale
        \param str string in which to do removal
    */
    void remove_whitespace(std::string& str);
    
    //! Create a copy of a string with all whitespace characters removed
    /*! Create a copy of a string with  occurences of space, carriage return, 
        line feed, tab etc as defined by current locale removed
        \param str string in which to do removal
        \param str output recipient of copy
    */
    void remove_whitespace(const std::string& input, 
            std::string& output);

    //! Convert string to upper case
    /*! Convert string to upper case using current locale
        \param str string to convert
    */
    void toupper(std::string& str);

    //! Convert string to lower case
    /*! Convert string to lower case using current locale
        \param str string to convert
     */
    void tolower(std::string& str);

  } // namespace Utilities
} // namespace CoCoSt

#endif // COCOST_STRING_UTILS_H_

/*

Copyright (c) 2010-2011, Orjan Westin
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

The name of the contributor(s) may be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

